<?php
namespace Teknisa\COZ\Authentication;
use Teknisa\Libs\Util\Environment;
use Zeedhi\Framework\DTO;

class Authenticator{

    protected $libsValidations;
    protected $environment;
    protected $crudService;
    protected $routesToValidate;
    const INVALID_CENTRAL = 'AUTH_INVALID_CENTRAL';

    public function __construct($libsValidations, $environment, $crudService, $routesToValidate){
        $this->libsValidations  = $libsValidations;
        $this->environment  = $environment;
        $this->crudService = $crudService;
        $this->routesToValidate = $routesToValidate;
    }

    public function run($request){
        $this->libsValidations->callAuthenticator($request, false);
        if(array_search($request->getRoutePath(), $this->routesToValidate) !== false) $this->validateCentral($request);
    }
    
    private function validateCentral($request){
        if($request instanceof DTO\Request\Row){
            $row = $request->getRow();
            if(!$this->isValidCentral($row)) throw new \Exception(self::INVALID_CENTRAL);
           
        } else if($request instanceof DTO\Request\DataSet){
            $rows = $request->getDataSet()->getRows();
            $newRows = array();
            foreach($rows as $cRow) {
               if(!$this->isValidCentral($cRow)) throw new \Exception(self::INVALID_CENTRAL);
              
            }
        } else if($request instanceof DTO\Request\Filter){
            $conditions = $request->getFilterCriteria()->getConditions();
            $newConditions = array();
            foreach($conditions as $filter){
                if(!$this->isValidCentral($filter, true)) throw new \Exception(self::INVALID_CENTRAL);
            }
        }
    }
    
    private function isValidCentral($row, $isFilter = false){
        if($isFilter) {
            $cdcentral = ($row['columnName'] == 'CDCENTRAL' || $row['columnName'] == 'P_CDCENTRAL') ? $row['value'] : null;
            if($cdcentral == null) return true;
        }else {
            $cdcentral = null;
            if(array_key_exists('CDCENTRAL', $row)) $cdcentral = $row['CDCENTRAL'];
            else if(array_key_exists('P_CDCENTRAL', $row)) $cdcentral = $row['P_CDCENTRAL'];
        }

        $rowsItem = $this->findValidCentrais();
        
        $validCentrais = array_map(function($row){
            return $row['CDFILIAL'];
        }, $rowsItem);
        
        return array_search($cdcentral, $validCentrais) !== false;
    }
    
    private function findValidCentrais(){
        $dataSourceName = 'filialcentral';
    	$conditions = array(
        	array(
    	    	"columnName" => "NRORG",
    			"operator" => "=",
    			"value" => $this->environment->getNrOrgTrab()
        	    ),
        	array(
    	    	"columnName" => "CDOPERADOR",
    			"operator" => "=",
    			"value" => $this->environment->getCdOperador()
        	    )
    	);
    	
		$dataSet = $this->crudService->getFilteredDataSet($dataSourceName, $conditions);
        return $dataSet->getRows();
    }
}

