<?php
namespace Teknisa\COZ\Services;

use Doctrine\ORM\EntityManager;
use \Setforms2\Shared\ORM\DateTime;
use \Teknisa\Model\Planejamento\Entities\Celulaprod;

class CelulaprodServicoFilialTipopratService {

    const SQL_REMOVE_CELULA_PRODUCAO = "DELETE FROM CELULAPROD WHERE NRORG = ? AND DTPRODUCAO = ? AND NRDIASEMANA = ? AND CDFILIAGREGA = ? AND(INSTR(?, CDSERVFILI) > 0) AND (INSTR(?, CONCAT(CDCENTRAL,CDTIPOPRAT)) > 0 OR ? IS NULL)";
    const SQL_REMOVE_CELULA_PRODUCAO_POR_SERVICO = "DELETE FROM CELULAPROD WHERE NRORG = ? AND CDFILIAGREGA = ? AND (INSTR(?, CDSERVFILI) > 0) ";

    protected $util;
    protected $dataSourceManager;
    protected $environment;
    protected $entityManager;
    protected $novoCodigo;
    
    public function __construct($util, $dataSourceManager, $environment, $entityManager, $novoCodigo) {
        $this->util = $util;
        $this->dataSourceManager = $dataSourceManager;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->novoCodigo = $novoCodigo;
    }

    public function iniciaConexao() {
        $this->entityManager->getConnection()->beginTransaction();
    }

    public function realizaCommit() {
        $this->entityManager->getConnection()->commit();
    }

    public function realizaRollback() {
        $this->entityManager->getConnection()->rollback();
    }

    public function excluiCelulaProducao($nrorg, $dtproducao, $nrdiasemana, $cdfiliagrega, $cdservfili, $centraltpprat){
        $st = $this->entityManager->getConnection()->prepare(self::SQL_REMOVE_CELULA_PRODUCAO);
        $params = array($nrorg, $dtproducao, $nrdiasemana, $cdfiliagrega, $cdservfili, $centraltpprat, $centraltpprat);
        $st->execute($params);
    }

    public function excluiCelulaProducaoPorServico($nrorg, $cdfiliagrega, $cdservfili){
        $st = $this->entityManager->getConnection()->prepare(self::SQL_REMOVE_CELULA_PRODUCAO_POR_SERVICO);
        $params = array($nrorg, $cdfiliagrega, $cdservfili);
        $st->execute($params);
    }

    public function salvaCelulaProducao(
        $nrorg, $cdcentral, $dtproducao, $nrdiasemana, $cdtipoprat,
        $cdfiliagrega, $cdservfili
    ){
        try {
            date_default_timezone_set("America/Sao_Paulo");

            $celulaprod = $this->retornaCelulaProducao($nrorg, $dtproducao, $nrdiasemana, $cdtipoprat, $cdfiliagrega, $cdservfili);

            if(!$celulaprod) {
                $celulaprod = new Celulaprod();
                $celulaprod->setCdcentral($cdcentral);
                $celulaprod->setDtproducao(DateTime::createFromFormat('!d/m/Y', $dtproducao));
                $celulaprod->setNrdiasemana($nrdiasemana);
                $celulaprod->setCdtipoprat($cdtipoprat);
                $celulaprod->setCdfiliagrega($cdfiliagrega);
                $celulaprod->setCdservfili($cdservfili);

                $celulaprod->setNrorg($nrorg);
                $celulaprod->setNrorginclusao($nrorg);
                $celulaprod->setNrorgultatu($nrorg);
                $celulaprod->setDtinclusao(new DateTime());
                $celulaprod->setDtultatu(new DateTime());
                $celulaprod->setCdoperinclusao($this->environment->getCdOperador());
                $celulaprod->setCdoperultatu($this->environment->getCdOperador());
                $this->entityManager->persist($celulaprod);
                $this->entityManager->flush();

                return true;
            } else {
                return false;
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function retornaCelulaProducao($nrorg, $dtproducao, $nrdiasemana, $cdtipoprat, $cdfiliagrega, $cdservfili) {
        $celulaprod = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Celulaprod')
            ->findOneBy(array('nrorg' => $nrorg, 'dtproducao' => DateTime::createFromFormat('!d/m/Y', $dtproducao), 'nrdiasemana' => $nrdiasemana, 'cdtipoprat' => $cdtipoprat, 'cdfiliagrega' => $cdfiliagrega, 'cdservfili' => $cdservfili));
        return $celulaprod;
    }
    
    public function getEnvironment() {
        return $this->environment;
    }
}